Attribute VB_Name = "Main"
'   Copyright (c) 1999 by Microsoft Corporation
'
'   THE INFORMATION AND CODE PROVIDED HEREUNDER (COLLECTIVELY REFERRED TO
'   AS "SOFTWARE") IS PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER
'   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
'   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN
'   NO EVENT SHALL MICROSOFT CORPORATION OR ITS SUPPLIERS BE LIABLE FOR
'   ANY DAMAGES WHATSOEVER INCLUDING DIRECT, INDIRECT, INCIDENTAL,
'   CONSEQUENTIAL, LOSS OF BUSINESS PROFITS OR SPECIAL DAMAGES, EVEN IF
'   MICROSOFT CORPORATION OR ITS SUPPLIERS HAVE BEEN ADVISED OF THE
'   POSSIBILITY OF SUCH DAMAGES. SOME STATES DO NOT ALLOW THE EXCLUSION OR
'   LIMITATION OF LIABILITY FOR CONSEQUENTIAL OR INCIDENTAL DAMAGES SO THE
'   FOREGOING LIMITATION MAY NOT APPLY.
'
'   This software may be copied and distributed royalty-free subject to
'   the following conditions:
'
'   1. You must copy all software without modification and must include
'      all pages, if the software is distributed without inclusion in your
'      software product. If you are incorporating the software in
'      conjunction with and as a part of your software product which adds
'      substantial value, you may modify and include portions of the
'      software.
'
'   2. You must place all copyright notices and other protective
'      disclaimers and notices contained on the software on all copies of
'      the software and your software product.
'
'   3. Unless the software is incorporated in your software product which
'      adds substantial value, you may not distribute this software for
'      profit.
'
'   4. You may not use Microsoft's name, logo, or trademarks to market
'      your software product.
'
'   5. You agree to indemnify, hold harmless, and defend Microsoft and its
'      suppliers from and against any claims or lawsuits, including
'      attorneys' fees, that arise or result from the use or distribution
'      of your software product and any modifications to the software.

' ***
' *** Note: this is the version that requires Microsoft PowerPoint 2000
' ***

' *** to use more than six characters as narrators in your presentation, change the numbers in the following five lines
' *** yes, this is zero-based and actually allows seven characters to be used but I've simplified the scripting to begin counting the first character as number one
Public Narrator(6) As IAgentCtlCharacterEx
Public bNarratorLoaded(6) As Boolean
Public objLoopRequest(6) As Object
Public iLoopDuration(6) As Integer
Public bLoopContinue(6) As Boolean
Public AgentCtl As Agent
Public iCharacter As Integer
Public objRequest As Object
Public iSlideToProcess As Integer
Public iResumeSlide As Integer
Public iScreenHeight As Integer
Public iScreenWidth As Integer
Public iCurrentPos As Integer
Public iResumePos As Integer
Public strScript As String
Public bNarrationBegun As Boolean
Public bSuspended As Boolean
Public bProcessAnother As Boolean
Public bScriptedHide As Boolean

Dim classAgent As New clsAgent
Dim classPpt As New clsPpt

Declare Function GetSystemMetrics Lib "user32" (ByVal nIndex As Long) As Long

' begin narration of presentation, called from object action
Public Sub Narration()
    ' test if narrator loaded to prevent double trigger of mouse over event
    If Not bNarrationBegun Then
        bNarrationBegun = True
        ' initialize PowerPoint application object
        InitializeApp
        ' run once to connect to MS Agent
        Set AgentCtl = New Agent
        AgentCtl.Connected = True
        Set classAgent.AgentCtl = AgentCtl
        ' initialize screen dimensions using system metrics - SM_CXSCREEN: 0, SM_CYSCREEN: 1
        ' slide show view doesn't use full screen resolution so ActivePresentation.SlideShowWindow.Height is no use
        iScreenHeight = GetSystemMetrics(1)
        iScreenWidth = GetSystemMetrics(0)
        ' parse current slide
        iSlideToProcess = ActivePresentation.SlideShowWindow.View.Slide.SlideIndex
        ProcessNotesPage
    End If
End Sub

' begin narration of current slide from its Note page
Public Sub ProcessNotesPage()
    ' set variable to hold current slide number (for resume from different slide)
    iResumeSlide = ActivePresentation.SlideShowWindow.View.Slide.SlideIndex
    ' get text from Notes page
    strNotes = ActivePresentation.Slides(iSlideToProcess).NotesPage.Shapes.Placeholders(2).TextFrame.TextRange
    ' find magic marker preceding script
    iMarker = InStr(strNotes, "^*#{}#*^")
    If iMarker = 0 Then
        MsgBox "Scripting error. There is no scripting on this Notes page or it is incorrectly formatted. Please see the accompanying documentation on the proper scripting syntax and usage."
        End
    Else
        ' ignore text prior to script marker
        strScript = Mid(strNotes, iMarker + 8)
        ' parse out blank spaces on the marker line and any following blank lines
        ProcessStartofScript
        ' parse out blank lines and spaces at end of the script
        ProcessEndofScript
        ' initialize variable to hold next slide number to proceed to
        iSlideToProcess = 0
        ' initialize pointer to current position within strScript
        iCurrentPos = 1
        bProcessAnother = True
        While bProcessAnother And Not bSuspended
            ' parse and execute first line of this script
            bProcessAnother = ProcessLine()
        Wend
    End If
End Sub

' clean up any blank lines and spaces at beginning of script
Private Sub ProcessStartofScript()
    iFirstCommandPos = 0
    iCurrentPos = 1
    iEOL = 1
    ' work through each line of script to get to last command line
    While iFirstCommandPos = 0
        ' find length of line
        iEOL = InStr(iCurrentPos, strScript, Chr(10)) - iCurrentPos + 1
        ' not last line of script
        If iEOL > 0 Then
            ' strips leading and trailing spaces for command line
            strLine = Trim(Mid(strScript, iCurrentPos, iEOL - 2))
            ' only blank spaces on this line
            If Len(strLine) = 0 Then
                ' increment to next line
                iCurrentPos = iCurrentPos + iEOL
            Else
                ' mark this as the first command line
                iFirstCommandPos = iCurrentPos
                strScript = Mid(strScript, iFirstCommandPos)
            End If
        ' reached last line of script, still empty
        Else
            ' create exit condition, nothing in this script
            iFirstCommandPos = -1
            MsgBox "Scripting error. There is nothing to execute in the Notes page of this slide."
        End If
    Wend
End Sub

' clean up any blank lines and spaces at end of script
Private Sub ProcessEndofScript()
    iSlideLine = InStr(strScript, "SLIDE")
    iCurrentPos = iSlideLine
    ' if SLIDE found, check if it is commented out and find last uncommented SLIDE
    While iCurrentPos > 0
        i = 1
        While i > 0
            char = Mid(strScript, iSlideLine - i, 1)
            Select Case char
                ' current SLIDE line is also commented out
                Case "!"
                    ' disqualified - find next SLIDE line in script
                    iSlideLine = InStr(iSlideLine + 5, strScript, "SLIDE")
                    iCurrentPos = iSlideLine
                    i = 0
                ' reached end of previous line, valid SLIDE line so just exit
                Case Chr(10)
                    i = 0
                    iCurrentPos = 0
            End Select
        Wend
    Wend
    ' check SLIDE command line
    If iSlideLine > 0 Then
        iEOL = InStr(iSlideLine, strScript, Chr(10))
        ' if SLIDE line doesn't have EOL
        If iEOL = 0 Then
            strScript = strScript + Chr(13) + Chr(10)
        Else
            strScript = Mid(strScript, 1, iEOL)
        End If
    ' no SLIDE command line, SUSPEND after last command
    Else
        iLastCommandPos = 0
        iCurrentPos = 1
        iEOL = InStr(iCurrentPos, strScript, Chr(10))
        ' work through each line of script to get to last command line
        While iEOL > 0
            strLine = Trim(Mid(strScript, iCurrentPos, iEOL - iCurrentPos - 1))
            ' check if there is anything on trimmed line
            If Len(strLine) > 0 Then
                ' then mark this as the current last command line
                iLastCommandPos = iCurrentPos
            End If
            ' increment to next line
            iCurrentPos = iEOL + 1
            ' find EOL
            iEOL = InStr(iCurrentPos, strScript, Chr(10))
        Wend
        ' iCurrentPos on last line now
        ' if nothing on this last line, last command line has EOL
        If Len(Trim(Mid(strScript, iCurrentPos))) = 0 Then
            iEOL = InStr(iLastCommandPos, strScript, Chr(10))
            ' truncate everything after last command
            strScript = Mid(strScript, 1, iEOL)
        ' something on last line
        Else
            ' set pointers, add EOL
            iLastCommandPos = iCurrentPos
            strScript = strScript + Chr(13) + Chr(10)
        End If
        ' if last command line is not SUSPEND (END too?), add SUSPEND
        If Mid(LTrim(Mid(strScript, iLastCommandPos)), 1, 7) <> "SUSPEND" Then
            strScript = strScript + "SUSPEND" + Chr(13) + Chr(10)
        End If
    End If
End Sub

' processes and executes each command line
Public Function ProcessLine() As Integer
    ' set variable to resume processing at this line
    iResumePos = iCurrentPos
    iEOL = InStr(iCurrentPos, strScript, Chr(10))
    ' isolate command line from rest of script and CRLF
    strLine = Trim(Mid(strScript, iCurrentPos, iEOL - iCurrentPos - 1))
    ' check if current line is a comment line in the script
    If Mid(strLine, 1, 1) = "!" Then
        strAction = "COMMENT"
    ' not a comment line in the script
    Else
        iCharacter = Val(strLine)
        ' check if a character is specified, set character variable
        If iCharacter > 0 Then
            ' set string variable to remainder of command line
            strLine = LTrim(Mid(strLine, InStr(strLine, ")") + 1))
        Else
            iCharacter = 1
        End If
        ' check if next command line should be processed immediately
        If Right(strLine, 3) = "<+>" Then
            ProcessLine = True
            ' strip off next command line marker and any extra blank spaces
            strLine = RTrim(Mid(strLine, 1, Len(strLine) - 3))
        Else
            ProcessLine = False
        End If
        iActionLength = InStr(strLine, " ")
        ' check if command doesn't have parameters (e.g. Hide)
        If iActionLength = 0 Then
            ' no parameter for command of current line
            strAction = strLine
        Else
            ' parse verb of current line
            strAction = Mid(strLine, 1, iActionLength - 1)
            ' parse remainder of current line after verb
            strParms = LTrim(Mid(strLine, iActionLength + 1))
        End If
    End If
    ' *** Note: the following if...else...endif is Presentation Narrator 97 version specific code
    ' check if there are no Slide Shows still running
    If SlideShowWindows.Count = 0 Then
        Narrator(iCharacter).Hide
    Else
    ' loop through slide shows that are running
        For i = 1 To SlideShowWindows.Count
            ' check if this is the slide show for the active presentation
            If SlideShowWindows(i).Presentation.Name = ActivePresentation.Name Then
                bContinue = True
                Exit For
            End If
        Next
        ' if active presentation slide show is no longer running, hide character and end narration
        If Not bContinue Then
            Narrator(iCharacter).Hide
        End If
    End If
    Select Case strAction
        ' output text speech and captioning in word balloon
        Case "SAY"
            ' pass text to parse for TTS output
            strOutput = ParseTTSInput(strParms)
            ' set object variable if need to wait for completion
            If ProcessLine = False Then
                Set objRequest = Narrator(iCharacter).Speak(strOutput)
            ' else just proceed to next command line
            Else
                Narrator(iCharacter).Speak (strOutput)
            End If
        ' output text speech only, no captioning in word balloon
        Case "SAYNOCAP"
            If Narrator(iCharacter).Balloon.Style And 1 Then
                bResetCap = True
                Narrator(iCharacter).Balloon.Style = Narrator(iCharacter).Balloon.Style And (Not 1)
            End If
            ' pass text to parse for TTS output
            strOutput = ParseTTSInput(strParms)
            ' set object variable if need to wait for completion
            If ProcessLine = False Then
                Set objRequest = Narrator(iCharacter).Speak(strOutput)
            ' else just proceed to next command line
            Else
                Narrator(iCharacter).Speak (strOutput)
            End If
            If bResetCap Then
                Narrator(iCharacter).Balloon.Style = Narrator(iCharacter).Balloon.Style Or 1
            End If
        'output audio and captioning in word balloon
        Case "AUDIO"
            iFileLength = InStr(2, strParms, ">")
            ' audio file is incorrectly specified - format is: AUDIO <filespec> text
            If iFileLength = 0 Then
                MsgBox " Authoring error: Incorrect specification of file for AUDIO command in script."
            End If
            ' set variable to filespec
            strFile = Trim(Mid(strParms, 2, iFileLength - 2))
            ' set variable to text to display
            strText = Trim(Mid(strParms, iFileLength + 1))
            ' set object variable if need to wait for completion
            If ProcessLine = False Then
                Set objRequest = Narrator(iCharacter).Speak(strText, strFile)
            ' else just proceed to next command line
            Else
                Narrator(iCharacter).Speak strText, strFile
            End If
        ' output audio only, no captioning in word balloon
        Case "AUDIONOCAP"
            iFileLength = InStr(2, strParms, ">")
            ' audio file is incorrectly specified - format is: AUDIO <filespec> text
            If iFileLength = 0 Then
                MsgBox " Authoring error: Incorrect specification of file for AUDIO command in script."
            End If
            ' set variable to filespec
            strFile = Trim(Mid(strParms, 2, iFileLength - 2))
            ' set object variable if need to wait for completion
            If ProcessLine = False Then
                Set objRequest = Narrator(iCharacter).Speak("", strFile)
            ' else just proceed to next command line
            Else
                Narrator(iCharacter).Speak "", strFile
            End If
        ' displays the specified text in a thought word balloon
        Case "THINK"
            ' set variable to animation name
            strText = Trim(strParms)
            ' set object variable if need to wait for completion
            If ProcessLine = False Then
                Set objRequest = Narrator(iCharacter).Think(strText)
            ' else just proceed to next command line
            Else
                Narrator(iCharacter).Think (strText)
            End If
        ' play animation
        Case "PLAY"
            ' set variable to animation name
            strAnimation = Trim(strParms)
            ' set object variable if need to wait for completion
            If ProcessLine = False Then
                Set objRequest = Narrator(iCharacter).Play(strAnimation)
            ' else just proceed to next command line
            Else
                Narrator(iCharacter).Play (strAnimation)
            End If
        ' move to screen location (as percent of resolution)
        Case "MOVE"
            x = Val(strParms) * iScreenWidth / 100
            y = Val(Mid(strParms, InStr(strParms, ",") + 1)) * iScreenHeight / 100
            ' set object variable if need to wait for completion
            If ProcessLine = False Then
                Set objRequest = Narrator(iCharacter).MoveTo(x, y)
            ' else just proceed to next command line
            Else
                Narrator(iCharacter).MoveTo x, y
            End If
        ' play looped animation for duration (in seconds)
        Case "LOOP"
            iLoopDuration(iCharacter) = Val(strParms)
            strAnimation = Trim(Mid(strParms, InStr(strParms, "=") + 1))
            ' if not immediate execute mode, need to set flag to continue to next command line after loop
            If ProcessLine = False Then
                bLoopContinue(iCharacter) = True
            ' immediate execute mode, don't set flag
            Else
                bLoopContinue(iCharacter) = False
            End If
            ' set object variable to stop loop regardless of mode
            Set objLoopRequest(iCharacter) = Narrator(iCharacter).Play(strAnimation)
        ' go to specified slide upon completion of next command
        Case "SLIDE"
            ' set slide to go to next
            iSlideToProcess = Val(strParms)
            ' return true to process last command of this slide
            ProcessLine = True
        ' load and display narrator
        Case "SHOW"
            iComma = InStr(strParms, ",")
            strCharacterName = Trim(Mid(strParms, 1, iComma - 1))
            strParms = LTrim(Mid(strParms, iComma + 1))
            iComma = InStr(strParms, ",")
            strCharacterFile = Trim(Mid(strParms, 1, iComma - 1))
            strParms = LTrim(Mid(strParms, iComma + 1))
            x = Val(strParms)
            y = Val(Mid(strParms, InStr(strParms, ",") + 1))
            ' check if already loaded for cases of looping scripts
            If Not bNarratorLoaded(iCharacter) Then
                AgentCtl.Characters.Load strCharacterName, strCharacterFile
                Set Narrator(iCharacter) = AgentCtl.Characters(strCharacterName)
                bNarratorLoaded(iCharacter) = True
                ' this sets the language for the character to U.S. English
                Narrator(iCharacter).LanguageID = &H409
                Set cmdSuspend = Narrator(iCharacter).Commands.Add("Suspend", "Suspend", "Suspend", True, True)
                Set cmdResume = Narrator(iCharacter).Commands.Add("Resume", "Resume", "Resume", False, False)
            End If
            Narrator(iCharacter).MoveTo x * iScreenWidth / 100, y * iScreenHeight / 100
            ' set object variable if need to wait for completion
            If ProcessLine = False Then
                Set objRequest = Narrator(iCharacter).Show
            ' else just proceed to next command line
            Else
                Narrator(iCharacter).Show
            End If
        ' suspend processing for duration (in seconds)
        Case "WAIT"
            TimerLoop Val(strParms)
            ' do another line if WAIT command is not last line of NotePage
            ProcessLine = True
        ' dismiss narrator
        Case "HIDE"
            ' set so narration doesn't necessarily end in narrator Hide event
            bScriptedHide = True
            ' set object variable if need to wait for completion
            If ProcessLine = False Then
                Set objRequest = Narrator(iCharacter).Hide
            ' else just proceed to next command line
            Else
                Narrator(iCharacter).Hide
            End If
        ' dismiss narrator
        Case "END"
            ' don't continue to process another command line
            ProcessLine = False
            ' set to end narration in narrator Hide event
            bScriptedHide = False
            Narrator(iCharacter).Hide
        ' suspend the narration - different than PAUSE output tag
        Case "SUSPEND"
            ' don't continue to process another command line
            ProcessLine = False
            bSuspended = True
            ' reset context menu items
            SetSuspendedMenu
        ' comment line, no action, do another line
        Case "COMMENT"
            ProcessLine = True
        ' basic shell command to execute another application
        Case "RUN"
            ' application will open with focus and its most recent size and position
            varReturn = Shell(strParms, vbNormalFocus)
            ' continue to process another command line
            ProcessLine = True
        ' advance slide
        Case "NEXT"
            ActivePresentation.SlideShowWindow.View.Next
            ProcessLine = True
        ' turn character's sound effects off
        Case "SOUNDFXOFF"
            Narrator(iCharacter).SoundEffectsOn = False
            ProcessLine = True
        ' turn character's sound effects on
        Case "SOUNDFXON"
            Narrator(iCharacter).SoundEffectsOn = True
            ProcessLine = True
        'valid case of blank line, process another command line
        Case ""
            ProcessLine = True
        ' error
        Case Else
            MsgBox "Unrecognized command in narration script. Only valid commands can be used. Please refer to documentation or check for typing errors."
    End Select
    ' if SLIDE command, process its action to complete
    If strAction = "SLIDE" Then
        iEqualSign = InStr(Mid(strScript, iCurrentPos), "=")
        ' check for required "=" character on this line
        If iEqualSign > 0 Then
            ' set variable to advance iCurrentPos to command for SLIDE to complete
            iCurrentPos = iCurrentPos + iEqualSign
        ' syntax error condition
        Else
            MsgBox "Syntax error: The SLIDE command line for this script is missing the '=' character"
            End
        End If
    Else
        ' continue processing next line
        iCurrentPos = iEOL + 1
    End If
End Function

' parse text for TTS output
Private Function ParseTTSInput(ByVal strInput As String) As String
    ' parse map speech output tags
    Do While InStr(strInput, "{") <> 0
        iLeftBracket = InStr(strInput, "{")
        iEqualSign = InStr(iLeftBracket + 1, strInput, "=")
        iRightBracket = InStr(iEqualSign + 1, strInput, "}")
        strPhonetic = Mid(strInput, iLeftBracket + 1, iEqualSign - iLeftBracket - 1)
        strLiteral = Mid(strInput, iEqualSign + 1, iRightBracket - iEqualSign - 1)
        strParse = Mid(strInput, 1, iLeftBracket - 1) + "\map=" + Chr(34) + strPhonetic + Chr(34) + "="
        strParse = strParse + Chr(34) + strLiteral + Chr(34) + "\" + Mid(strInput, iRightBracket + 1)
        strInput = strParse
    Loop
    ' parse emphasis speech output tags
    Do While InStr(strInput, "^EMPHASIS") <> 0
        ' set pointer to emphasis tag
        iMarker = InStr(strInput, "^EMPHASIS")
        strInput = Mid(strInput, 1, iMarker - 1) + "\emp\" + Mid(strInput, InStr(Mid(strInput, iMarker + 9), "^") + iMarker + 9)
    Loop
    ' parse pitch speech output tags
    Do While InStr(strInput, "^PITCH") <> 0
        ' set pointer to pitch tag
        iMarker = InStr(strInput, "^PITCH")
        iPitch = Val(Mid(strInput, iMarker + 7))
        strInput = Mid(strInput, 1, iMarker - 1) + "\Pit=" + Mid(Str(iPitch), 2) + "\" + Mid(strInput, InStr(Mid(strInput, iMarker + 7), "^") + iMarker + 7)
    Loop
    ' parse speed speech output tags
    Do While InStr(strInput, "^SPEED") <> 0
        iMarker = InStr(strInput, "^SPEED")
        iSpeed = Val(Mid(strInput, iMarker + 7))
        strInput = Mid(strInput, 1, iMarker - 1) + "\Spd=" + Mid(Str(iSpeed), 2) + "\" + Mid(strInput, InStr(Mid(strInput, iMarker + 7), "^") + iMarker + 7)
    Loop
    ' parse volume speech output tags
    Do While InStr(strInput, "^VOLUME") <> 0
        ' set pointer to volume tag
        iMarker = InStr(strInput, "^VOLUME")
        iVolume = Val(Mid(strInput, iMarker + 8))
        strInput = Mid(strInput, 1, iMarker - 1) + "\Vol=" + Mid(Str(iVolume), 2) + "\" + Mid(strInput, InStr(Mid(strInput, iMarker + 8), "^") + iMarker + 8)
    Loop
    ' parse reset speech output tags
    Do While InStr(strInput, "^RESET") <> 0
        ' set pointer to reset tag
        iMarker = InStr(strInput, "^RESET")
        strInput = Mid(strInput, 1, iMarker - 1) + "\Rst\" + Mid(strInput, InStr(Mid(strInput, iMarker + 6), "^") + iMarker + 6)
    Loop
    ' parse pause speech output tags
    Do While InStr(strInput, "^PAUSE") <> 0
        iMarker = InStr(strInput, "^PAUSE")
        iPause = Val(Mid(strInput, iMarker + 7))
        strInput = Mid(strInput, 1, iMarker - 1) + "\Pau=" + Mid(Str(iPause), 2) + "\" + Mid(strInput, InStr(Mid(strInput, iMarker + 7), "^") + iMarker + 7)
    Loop
    ' parse monotone speech output tags
    Do While InStr(strInput, "^MONOTONE") <> 0
        iMarker = InStr(strInput, "^MONOTONE")
        strInput = Mid(strInput, 1, iMarker - 1) + "\Chr=" + Chr(34) + "Monotone" + Chr(34) + "\" + Mid(strInput, InStr(Mid(strInput, iMarker + 9), "^") + iMarker + 9)
    Loop
    ' parse whisper speech output tags
    Do While InStr(strInput, "^WHISPER") <> 0
        iMarker = InStr(strInput, "^WHISPER")
        strInput = Mid(strInput, 1, iMarker - 1) + "\Chr=" + Chr(34) + "Whisper" + Chr(34) + "\" + Mid(strInput, InStr(Mid(strInput, iMarker + 8), "^") + iMarker + 8)
    Loop
    ' parse normal voice character speech output tags
    Do While InStr(strInput, "^NORMAL") <> 0
        iMarker = InStr(strInput, "^NORMAL")
        strInput = Mid(strInput, 1, iMarker - 1) + "\Chr=" + Chr(34) + "Normal" + Chr(34) + "\" + Mid(strInput, InStr(Mid(strInput, iMarker + 7), "^") + iMarker + 7)
    Loop
    ' parse Address context speech output tags
    Do While InStr(strInput, "^ADDRESS") <> 0
        iMarker = InStr(strInput, "^ADDRESS")
        strInput = Mid(strInput, 1, iMarker - 1) + "\Ctx=" + Chr(34) + "Address" + Chr(34) + "\" + Mid(strInput, InStr(Mid(strInput, iMarker + 8), "^") + iMarker + 8)
    Loop
    ' parse E-mail context speech output tags
    Do While InStr(strInput, "^E-MAIL") <> 0
        iMarker = InStr(strInput, "^E-MAIL")
        strInput = Mid(strInput, 1, iMarker - 1) + "\Ctx=" + Chr(34) + "E-mail" + Chr(34) + "\" + Mid(strInput, InStr(Mid(strInput, iMarker + 7), "^") + iMarker + 7)
    Loop
    ' parse default context speech output tags
    Do While InStr(strInput, "^NOCONTEXT") <> 0
        iMarker = InStr(strInput, "^NOCONTEXT")
        strInput = Mid(strInput, 1, iMarker - 1) + "\Ctx=" + Chr(34) + "Unknown" + Chr(34) + "\" + Mid(strInput, InStr(Mid(strInput, iMarker + 10), "^") + iMarker + 10)
    Loop
    ' parse bookmark speech output tags
    ' NOTE: nothing can be done with these events from the script syntax
    ' NOTE: you can add code in the corresponding bookmark event handler for custom actions
    Do While InStr(strInput, "^MARK") <> 0
        ' set pointer to MARK tag
        iMarker = InStr(strInput, "^MARK")
        iBookmark = Val(Mid(strInput, iMarker + 6))
        strInput = Mid(strInput, 1, iMarker - 1) + "\Mrk=" + Mid(Str(iBookmark), 2) + "\" + Mid(strInput, InStr(Mid(strInput, iMarker + 6), "^") + iMarker + 6)
    Loop
    ' return parsed input text
    ParseTTSInput = strInput
End Function

' simple timer
Public Sub TimerLoop(ByVal Delay As Integer)
    ' initialize current time
    Start = Timer
    ' loop until duration of Delay has passed
    Do While Timer < Start + Delay
        DoEvents
    Loop
End Sub

' disable suspend menu item, enable resume menu item
Public Sub SetSuspendedMenu()
' *** to use more than six characters as narrators in your presentation, change the upper bound (number) in the following lines
    For i = 1 To 6
        ' if the character is loaded, change its context menu items
        If bNarratorLoaded(i) Then
            Narrator(i).Commands("Suspend").Enabled = False
            Narrator(i).Commands("Suspend").Visible = False
            Narrator(i).Commands("Resume").Enabled = True
            Narrator(i).Commands("Resume").Visible = True
        End If
    Next
End Sub

' disable resume menu item, enable suspend menu item
Public Sub SetResumedMenu()
' *** to use more than six characters as narrators in your presentation, change the upper bound (number) in the following line
    For i = 1 To 6
        ' if the character is loaded, change its context menu items
        If bNarratorLoaded(i) Then
            Narrator(i).Commands("Resume").Enabled = False
            Narrator(i).Commands("Resume").Visible = False
            Narrator(i).Commands("Suspend").Enabled = True
            Narrator(i).Commands("Suspend").Visible = True
        End If
    Next
End Sub

' creates connection between this code and Microsoft PowerPoint
Sub InitializeApp()
    Set classPpt.App = Application
End Sub
